@@ -1,7 +1,7 @@
 /* pdftoepdf.cc
 
    Copyright 1996-2006 Han The Thanh <thanh@pdftex.org>
-   Copyright 2006-2009 Taco Hoekwater <taco@luatex.org>
+   Copyright 2006-2010 Taco Hoekwater <taco@luatex.org>
 
    This file is part of LuaTeX.
 
@@ -19,42 +19,8 @@
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
 static const char _svn_version[] =
-    "$Id: pdftoepdf.cc 3257 2009-12-15 23:19:55Z hhenkel $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/image/pdftoepdf.cc $";
-
-#include <stdlib.h>
-#include <math.h>
-#include <stddef.h>
-#include <stdio.h>
-#include <string.h>
-#include <ctype.h>
-#include <sys/stat.h>
-#ifdef POPPLER_VERSION
-#  define GString GooString
-#  include <dirent.h>
-#  include <poppler-config.h>
-#  include <goo/GooString.h>
-#  include <goo/gmem.h>
-#  include <goo/gfile.h>
-#else
-#  include <aconf.h>
-#  include <GString.h>
-#  include <gmem.h>
-#  include <gfile.h>
-#  include <assert.h>
-#endif
-#include "Object.h"
-#include "Stream.h"
-#include "Array.h"
-#include "Dict.h"
-#include "XRef.h"
-#include "Catalog.h"
-#include "Link.h"
-#include "Page.h"
-#include "GfxFont.h"
-#include "PDFDoc.h"
-#include "GlobalParams.h"
-#include "Error.h"
+    "$Id: pdftoepdf.cc 3431 2010-02-17 20:35:50Z oneiros $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.60.0/source/texk/web2c/luatexdir/image/pdftoepdf.cc $";
 
 #include "epdf.h"
 
@@ -68,80 +34,13 @@ static const char _svn_version[] =
 
 #define pdfkeyprefix "PTEX"
 
-// PdfObject encapsulates the xpdf Object type,
-// and properly frees its resources on destruction.
-// Use obj-> to access members of the Object,
-// and &obj to get a pointer to the object.
-// It is no longer necessary to call Object::free explicitely.
-
-/* *INDENT-OFF* */
-class PdfObject {
-  public:
-    PdfObject() {               // nothing
-    }
-    ~PdfObject() {
-        iObject.free();
-    }
-    Object *operator->() {
-        return &iObject;
-    }
-    Object *operator&() {
-        return &iObject;
-    }
-  private:                     // no copying or assigning
-    PdfObject(const PdfObject &);
-    void operator=(const PdfObject &);
-  public:
-    Object iObject;
-};
-/* *INDENT-ON* */
-
-// When copying the Resources of the selected page, all objects are
-// copied recursively top-down.  The findObjMap() function checks if an
-// object has already been copied; if so, instead of copying just the
-// new object number will be referenced.  The ObjMapTree guarantees,
-// that during the entire LuaTeX run any object from any embedded PDF
-// file will end up max. once in the output PDF file.  Indirect objects
-// are not fetched during copying, but get a new object number from
-// LuaTeX and then will be appended into a linked list.
-
-enum InObjType { objFont, objFontDesc, objOther };
-
-struct InObj {
-    Ref ref;                    // ref in original PDF
-    InObjType type;             // object type
-    integer num;                // new object number in output PDF
-    fd_entry *fd;               // pointer to /FontDescriptor object structure
-    integer enc_objnum;         // Encoding for objFont
-    InObj *next;                // next entry in list of indirect objects
-};
-
-struct UsedEncoding {
-    integer enc_objnum;
-    GfxFont *font;
-    UsedEncoding *next;
-};
-
-static UsedEncoding *encodingList;
 static GBool isInit = gFalse;
-static bool groupIsIndirect;
-static PdfObject lastGroup;
 
 //**********************************************************************
-// Maintain AVL tree of open embedded PDF files
+// Maintain AVL tree of all PDF files for embedding
 
 static avl_table *PdfDocumentTree = NULL;
 
-struct PdfDocument {
-    char *file_path;            // full file name including path
-    char *checksum;             // for reopening
-    PDFDoc *doc;
-    XRef *xref;
-    InObj *inObjList;
-    avl_table *ObjMapTree;
-    int occurences;             // number of references to the PdfDocument; it can be deleted when occurences == 0
-};
-
 // AVL sort PdfDocument into PdfDocumentTree by file_path
 
 static int CompPdfDocument(const void *pa, const void *pb, void * /*p */ )
@@ -168,7 +67,7 @@ static PdfDocument *findPdfDocument(char *file_path)
 
 char *get_file_checksum(char *a);
 
-static PdfDocument *refPdfDocument(char *file_path)
+PdfDocument *refPdfDocument(char *file_path)
 {
     PdfDocument *pdf_doc = findPdfDocument(file_path);
     if (pdf_doc == NULL) {
@@ -188,7 +87,6 @@ static PdfDocument *refPdfDocument(char *file_path)
         pdf_doc->doc = new PDFDoc(docName);     // takes ownership of docName
         if (!pdf_doc->doc->isOk() || !pdf_doc->doc->okToPrint())
             pdftex_fail("xpdf: reading PDF image failed");
-        pdf_doc->xref = NULL;
         pdf_doc->inObjList = NULL;
         pdf_doc->occurences = 0;        // 0 = unreferenced
     }
@@ -205,6 +103,8 @@ static PdfDocument *refPdfDocument(char *file_path)
 }
 
 //**********************************************************************
+// AVL sort ObjMap into ObjMapTree by object number and generation
+
 // keep the ObjMap struct small, as these are accumulated until the end
 
 struct ObjMap {
@@ -212,8 +112,6 @@ struct ObjMap {
     int out_num;                // object num after embedding (gen == 0)
 };
 
-// AVL sort ObjMap into ObjMapTree by object number and generation
-
 static int CompObjMap(const void *pa, const void *pb, void * /*p */ )
 {
     const Ref *a = &(((const ObjMap *) pa)->in);
@@ -253,62 +151,29 @@ static void addObjMap(PdfDocument * pdf_doc, Ref in, int out_num)
     assert(aa != NULL);
 }
 
-//**********************************************************************
+// When copying the Resources of the selected page, all objects are
+// copied recursively top-down.  The findObjMap() function checks if an
+// object has already been copied; if so, instead of copying just the
+// new object number will be referenced.  The ObjMapTree guarantees,
+// that during the entire LuaTeX run any object from any embedded PDF
+// file will end up max. once in the output PDF file.  Indirect objects
+// are not fetched during copying, but get a new object number from
+// LuaTeX and then will be appended into a linked list.
 
-// Replacement for
-//      Object *initDict(Dict *dict1){ initObj(objDict); dict = dict1; return this; }
-
-static void initDictFromDict(PdfDocument * pdf_doc, PdfObject & obj,
-                             Dict * dict)
-{
-    obj->initDict(pdf_doc->xref);
-    for (int i = 0, l = dict->getLength(); i < l; i++) {
-        Object obj1;
-        obj->dictAdd(copyString(dict->getKey(i)), dict->getValNF(i, &obj1));
-    }
-}
-
-static int addEncoding(GfxFont * gfont, int obj_num)
-{
-    UsedEncoding *n;
-    n = new UsedEncoding;
-    n->next = encodingList;
-    encodingList = n;
-    n->font = gfont;
-    n->enc_objnum = obj_num;
-    return n->enc_objnum;
-}
-
-#define addFont(ref, fd, enc_objnum) \
-    addInObj(pdf, pdf_doc, objFont, ref, fd, enc_objnum)
-
-// addFontDesc is only used to avoid writing the original FontDescriptor
-// from the PDF file.
-
-#define addFontDesc(ref, fd) \
-    addInObj(pdf, pdf_doc, objFontDesc, ref, fd, 0)
-
-#define addOther(ref) \
-    addInObj(pdf, pdf_doc, objOther, ref, NULL, 0)
-
-static int addInObj(PDF pdf, PdfDocument * pdf_doc, InObjType type, Ref ref,
-                    fd_entry * fd, integer e)
+static int addInObj(PDF pdf, PdfDocument * pdf_doc, Ref ref)
 {
     ObjMap *obj_map;
     InObj *p, *q, *n;
-    assert(ref.num != 0);
+    if (ref.num == 0) {
+        pdftex_fail("PDF inclusion: reference to invalid object"
+                    " (is the included pdf broken?)");
+    }
     if ((obj_map = findObjMap(pdf_doc, ref)) != NULL)
         return obj_map->out_num;
     n = new InObj;
     n->ref = ref;
-    n->type = type;
     n->next = NULL;
-    n->fd = fd;
-    n->enc_objnum = e;
-    if (type == objFontDesc)
-        n->num = get_fd_objnum(fd);
-    else
-        n->num = pdf_new_objnum(pdf);
+    n->num = pdf_new_objnum(pdf);
     addObjMap(pdf_doc, ref, n->num);
     if (pdf_doc->inObjList == NULL)
         pdf_doc->inObjList = n;
@@ -323,199 +188,8 @@ static int addInObj(PDF pdf, PdfDocument * pdf_doc, InObjType type, Ref ref,
     return n->num;
 }
 
-static void copyName(PDF pdf, char *s)
-{
-    pdf_puts(pdf, "/");
-    for (; *s != 0; s++) {
-        if (isdigit(*s) || isupper(*s) || islower(*s) || *s == '_' ||
-            *s == '.' || *s == '-' || *s == '+')
-            pdf_out(pdf, *s);
-        else
-            pdf_printf(pdf, "#%.2X", *s & 0xFF);
-    }
-}
-
-static int getNewObjectNumber(PdfDocument * pdf_doc, Ref ref)
-{
-    ObjMap *obj_map;
-    if ((obj_map = findObjMap(pdf_doc, ref)) != NULL)
-        return obj_map->out_num;
-    pdftex_fail("Object not yet copied: %i %i", ref.num, ref.gen);
-    return 0;
-}
-
-static void copyObject(PDF, PdfDocument *, Object *);
-
-static void copyDictEntry(PDF pdf, PdfDocument * pdf_doc, Object * obj, int i)
-{
-    PdfObject obj1;
-    copyName(pdf, obj->dictGetKey(i));
-    pdf_puts(pdf, " ");
-    obj->dictGetValNF(i, &obj1);
-    copyObject(pdf, pdf_doc, &obj1);
-    pdf_puts(pdf, "\n");
-}
-
-static void copyDict(PDF pdf, PdfDocument * pdf_doc, Object * obj)
-{
-    int i, l;
-    if (!obj->isDict())
-        pdftex_fail("PDF inclusion: invalid dict type <%s>",
-                    obj->getTypeName());
-    for (i = 0, l = obj->dictGetLength(); i < l; ++i)
-        copyDictEntry(pdf, pdf_doc, obj, i);
-}
-
-static void copyFontDict(PDF pdf, PdfDocument * pdf_doc, Object * obj,
-                         InObj * r)
-{
-    int i, l;
-    char *key;
-    if (!obj->isDict())
-        pdftex_fail("PDF inclusion: invalid dict type <%s>",
-                    obj->getTypeName());
-    pdf_puts(pdf, "<<\n");
-    assert(r->type == objFont); // FontDescriptor is in fd_tree
-    for (i = 0, l = obj->dictGetLength(); i < l; ++i) {
-        key = obj->dictGetKey(i);
-        if (strncmp("FontDescriptor", key, strlen("FontDescriptor")) == 0
-            || strncmp("BaseFont", key, strlen("BaseFont")) == 0
-            || strncmp("Encoding", key, strlen("Encoding")) == 0)
-            continue;           // skip original values
-        copyDictEntry(pdf, pdf_doc, obj, i);
-    }
-    // write new FontDescriptor, BaseFont, and Encoding
-    pdf_printf(pdf, "/FontDescriptor %d 0 R\n", (int) get_fd_objnum(r->fd));
-    pdf_printf(pdf, "/BaseFont %d 0 R\n", (int) get_fn_objnum(pdf, r->fd));
-    pdf_printf(pdf, "/Encoding %d 0 R\n", (int) r->enc_objnum);
-    pdf_puts(pdf, ">>");
-}
-
-static void copyStream(PDF pdf, Stream * str)
-{
-    int c;
-    str->reset();
-    while ((c = str->getChar()) != EOF) {
-        pdf_out(pdf, c);
-        pdf->last_byte = c;
-    }
-}
-
-static void copyProcSet(PDF pdf, Object * obj)
-{
-    int i, l;
-    PdfObject procset;
-    if (!obj->isArray())
-        pdftex_fail("PDF inclusion: invalid ProcSet array type <%s>",
-                    obj->getTypeName());
-    pdf_puts(pdf, "/ProcSet [ ");
-    for (i = 0, l = obj->arrayGetLength(); i < l; ++i) {
-        obj->arrayGetNF(i, &procset);
-        if (!procset->isName())
-            pdftex_fail("PDF inclusion: invalid ProcSet entry type <%s>",
-                        procset->getTypeName());
-        copyName(pdf, procset->getName());
-        pdf_puts(pdf, " ");
-    }
-    pdf_puts(pdf, "]\n");
-}
-
-#define REPLACE_TYPE1C true
-
-static void copyFont(PDF pdf, PdfDocument * pdf_doc, char *tag,
-                     Object * fontRef)
-{
-    ObjMap *obj_map;
-    PdfObject fontdict, subtype, basefont, fontdescRef, fontdesc, charset,
-        fontfile, ffsubtype, stemV;
-    GfxFont *gfont;
-    fd_entry *fd;
-    fm_entry *fontmap;
-    // Check whether the font has already been embedded before analysing it.
-    Ref ref = fontRef->getRef();
-    if ((obj_map = findObjMap(pdf_doc, ref)) != NULL) {
-        copyName(pdf, tag);
-        pdf_printf(pdf, " %d 0 R ", obj_map->out_num);
-        return;
-    }
-    // Only handle included Type1 (and Type1C) fonts; anything else will be copied.
-    // Type1C fonts are replaced by Type1 fonts, if REPLACE_TYPE1C is true.
-    if ((pdf->inclusion_copy_font != 0)
-        && fontRef->fetch(pdf_doc->xref, &fontdict)->isDict()
-        && fontdict->dictLookup((char *) "Subtype", &subtype)->isName()
-        && !strcmp(subtype->getName(), "Type1")
-        && fontdict->dictLookup((char *) "BaseFont", &basefont)->isName()
-        && fontdict->dictLookupNF((char *) "FontDescriptor",
-                                  &fontdescRef)->isRef()
-        && fontdescRef->fetch(pdf_doc->xref, &fontdesc)->isDict()
-        && (fontdesc->dictLookup((char *) "FontFile", &fontfile)->isStream()
-            || (REPLACE_TYPE1C
-                && fontdesc->dictLookup((char *) "FontFile3",
-                                        &fontfile)->isStream()
-                && fontfile->streamGetDict()->lookup((char *) "Subtype",
-                                                     &ffsubtype)->isName()
-                && !strcmp(ffsubtype->getName(), "Type1C")))
-        && (fontmap = lookup_fontmap(basefont->getName())) != NULL) {
-        // copy the value of /StemV
-        fontdesc->dictLookup((char *) "StemV", &stemV);
-        fd = epdf_create_fontdescriptor(fontmap, stemV->getInt(),
-                                        pdf_new_objnum(pdf));
-        if (fontdesc->dictLookup((char *) "CharSet", &charset)
-            && charset->isString() && is_subsetable(fontmap))
-            epdf_mark_glyphs(fd, charset->getString()->getCString());
-        else
-            embed_whole_font(fd);
-        addFontDesc(fontdescRef->getRef(), fd);
-        copyName(pdf, tag);
-        gfont = GfxFont::makeFont(pdf_doc->xref, tag, fontRef->getRef(),
-                                  fontdict->getDict());
-        pdf_printf(pdf, " %d 0 R ", addFont(fontRef->getRef(), fd,
-                                            addEncoding(gfont,
-                                                        pdf_new_objnum(pdf))));
-    } else {
-        copyName(pdf, tag);
-        pdf_puts(pdf, " ");
-        copyObject(pdf, pdf_doc, fontRef);
-    }
-}
-
-static void copyFontResources(PDF pdf, PdfDocument * pdf_doc, Object * obj)
-{
-    PdfObject fontRef;
-    int i, l;
-    if (!obj->isDict())
-        pdftex_fail("PDF inclusion: invalid font resources dict type <%s>",
-                    obj->getTypeName());
-    pdf_puts(pdf, "/Font << ");
-    for (i = 0, l = obj->dictGetLength(); i < l; ++i) {
-        obj->dictGetValNF(i, &fontRef);
-        if (fontRef->isRef() ||
-            (fontRef->isDict() && (pdf->inclusion_copy_font == 0)))
-            copyFont(pdf, pdf_doc, obj->dictGetKey(i), &fontRef);
-        else
-            pdftex_fail("PDF inclusion: invalid font in reference type <%s>",
-                        fontRef->getTypeName());
-    }
-    pdf_puts(pdf, ">>\n");
-}
-
-static void copyOtherResources(PDF pdf, PdfDocument * pdf_doc, Object * obj,
-                               char *key)
-{
-    // copies all other resources (write_epdf handles Fonts and ProcSets),
-    // but gives a warning if an object is not a dictionary.
-
-    if (!obj->isDict())
-        //FIXME: Write the message only to the log file
-        pdftex_warn("PDF inclusion: invalid other resource which is no dict"
-                    " (key '%s', type <%s>); copying it anyway.",
-                    key, obj->getTypeName());
-    copyName(pdf, key);
-    pdf_puts(pdf, " ");
-    copyObject(pdf, pdf_doc, obj);
-}
-
-// Function onverts double to string; very small and very large numbers
+//**********************************************************************
+// Function converts double to string; very small and very large numbers
 // are NOT converted to scientific notation.
 // n must be a number or real conforming to the implementation limits
 // of PDF as specified in appendix C.1 of the PDF Ref.
@@ -573,144 +247,171 @@ static char *convertNumToPDF(double n)
     return (char *) buf;
 }
 
-static void copyObject(PDF pdf, PdfDocument * pdf_doc, Object * obj)
+static void copyObject(PDF, PdfDocument *, Object *);
+
+static void copyString(PDF pdf, GString * string)
 {
-    PdfObject obj1;
-    int i, l, c;
-    Ref ref;
     char *p;
-    GString *s;
-    if (obj->isBool()) {
-        pdf_printf(pdf, "%s", obj->getBool()? "true" : "false");
-    } else if (obj->isInt()) {
-        pdf_printf(pdf, "%i", obj->getInt());
-    } else if (obj->isReal()) {
-        pdf_printf(pdf, "%s", convertNumToPDF(obj->getReal()));
-    } else if (obj->isNum()) {
-        pdf_printf(pdf, "%s", convertNumToPDF(obj->getNum()));
-    } else if (obj->isString()) {
-        s = obj->getString();
-        p = s->getCString();
-        l = s->getLength();
-        if (strlen(p) == (unsigned int) l) {
-            pdf_puts(pdf, "(");
-            for (; *p != 0; p++) {
-                c = (unsigned char) *p;
-                if (c == '(' || c == ')' || c == '\\')
-                    pdf_printf(pdf, "\\%c", c);
-                else if (c < 0x20 || c > 0x7F)
-                    pdf_printf(pdf, "\\%03o", c);
-                else
-                    pdf_out(pdf, c);
-            }
-            pdf_puts(pdf, ")");
-        } else {
-            pdf_puts(pdf, "<");
-            for (i = 0; i < l; i++) {
-                c = s->getChar(i) & 0xFF;
-                pdf_printf(pdf, "%.2x", c);
-            }
-            pdf_puts(pdf, ">");
+    unsigned char c;
+    size_t i, l;
+    p = string->getCString();
+    l = (size_t) string->getLength();
+    if (strlen(p) == l) {
+        pdf_puts(pdf, "(");
+        for (; *p != 0; p++) {
+            c = (unsigned char) *p;
+            if (c == '(' || c == ')' || c == '\\')
+                pdf_printf(pdf, "\\%c", c);
+            else if (c < 0x20 || c > 0x7F)
+                pdf_printf(pdf, "\\%03o", (int) c);
+            else
+                pdf_out(pdf, c);
         }
-    } else if (obj->isName()) {
-        copyName(pdf, obj->getName());
-    } else if (obj->isNull()) {
-        pdf_puts(pdf, "null");
-    } else if (obj->isArray()) {
-        pdf_puts(pdf, "[");
-        for (i = 0, l = obj->arrayGetLength(); i < l; ++i) {
-            obj->arrayGetNF(i, &obj1);
-            if (!obj1->isName())
-                pdf_puts(pdf, " ");
-            copyObject(pdf, pdf_doc, &obj1);
-        }
-        pdf_puts(pdf, "]");
-    } else if (obj->isDict()) {
-        pdf_puts(pdf, "<<\n");
-        copyDict(pdf, pdf_doc, obj);
-        pdf_puts(pdf, ">>");
-    } else if (obj->isStream()) {
-        initDictFromDict(pdf_doc, obj1, obj->streamGetDict());
-        obj->streamGetDict()->incRef();
-        pdf_puts(pdf, "<<\n");
-        copyDict(pdf, pdf_doc, &obj1);
-        pdf_puts(pdf, ">>\n");
-        pdf_puts(pdf, "stream\n");
-        copyStream(pdf, obj->getStream()->getUndecodedStream());
-        if (pdf->last_byte != '\n')
-            pdf_puts(pdf, "\n");
-        pdf_puts(pdf, "endstream");     // can't simply write pdf_end_stream()
-    } else if (obj->isRef()) {
-        ref = obj->getRef();
-        if (ref.num == 0) {
-            pdftex_fail
-                ("PDF inclusion: reference to invalid object"
-                 " (is the included pdf broken?)");
-        } else
-            pdf_printf(pdf, "%d 0 R", addOther(ref));
+        pdf_puts(pdf, ")");
     } else {
-        pdftex_fail("PDF inclusion: type <%s> cannot be copied",
-                    obj->getTypeName());
+        pdf_puts(pdf, "<");
+        for (i = 0; i < l; i++) {
+            c = (unsigned char) string->getChar(i);
+            pdf_printf(pdf, "%.2x", (int) c);
+        }
+        pdf_puts(pdf, ">");
     }
 }
 
+static void copyName(PDF pdf, char *s)
+{
+    pdf_puts(pdf, "/");
+    for (; *s != 0; s++) {
+        if (isdigit(*s) || isupper(*s) || islower(*s) || *s == '_' ||
+            *s == '.' || *s == '-' || *s == '+')
+            pdf_out(pdf, *s);
+        else
+            pdf_printf(pdf, "#%.2X", *s & 0xFF);
+    }
+}
+
+static void copyArray(PDF pdf, PdfDocument * pdf_doc, Array * array)
+{
+    int i, l;
+    PdfObject obj1;
+    pdf_puts(pdf, "[");
+    for (i = 0, l = array->getLength(); i < l; ++i) {
+        array->getNF(i, &obj1);
+        if (!obj1->isName())
+            pdf_puts(pdf, " ");
+        copyObject(pdf, pdf_doc, &obj1);
+    }
+    pdf_puts(pdf, "]");
+}
+
+static void copyDict(PDF pdf, PdfDocument * pdf_doc, Dict * dict)
+{
+    int i, l;
+    PdfObject obj1;
+    pdf_puts(pdf, "<<");
+    for (i = 0, l = dict->getLength(); i < l; ++i) {
+        copyName(pdf, dict->getKey(i));
+        pdf_puts(pdf, " ");
+        dict->getValNF(i, &obj1);
+        copyObject(pdf, pdf_doc, &obj1);
+        pdf_puts(pdf, "\n");
+    }
+    pdf_puts(pdf, ">>");
+}
+
+static void copyStreamStream(PDF pdf, Stream * str)
+{
+    int c;
+    str->reset();
+    while ((c = str->getChar()) != EOF) {
+        pdf_out(pdf, c);
+        pdf->last_byte = c;
+    }
+}
+
+static void copyStream(PDF pdf, PdfDocument * pdf_doc, Stream * stream)
+{
+    copyDict(pdf, pdf_doc, stream->getDict());
+    pdf_puts(pdf, "stream\n");
+    copyStreamStream(pdf, stream->getUndecodedStream());
+    if (pdf->last_byte != '\n')
+        pdf_puts(pdf, "\n");
+    pdf_puts(pdf, "endstream"); // can't simply write pdf_end_stream()
+}
+
+static void copyObject(PDF pdf, PdfDocument * pdf_doc, Object * obj)
+{
+    switch (obj->getType()) {
+    case objBool:
+        pdf_printf(pdf, "%s", obj->getBool()? "true" : "false");
+        break;
+    case objInt:
+        pdf_printf(pdf, "%i", obj->getInt());
+        break;
+    case objReal:
+        pdf_printf(pdf, "%s", convertNumToPDF(obj->getReal()));
+        break;
+        // not needed:
+        // GBool isNum() { return type == objInt || type == objReal; }
+    case objString:
+        copyString(pdf, obj->getString());
+        break;
+    case objName:
+        copyName(pdf, obj->getName());
+        break;
+    case objNull:
+        pdf_puts(pdf, "null");
+        break;
+    case objArray:
+        copyArray(pdf, pdf_doc, obj->getArray());
+        break;
+    case objDict:
+        copyDict(pdf, pdf_doc, obj->getDict());
+        break;
+    case objStream:
+        copyStream(pdf, pdf_doc, obj->getStream());
+        break;
+    case objRef:
+        pdf_printf(pdf, "%d 0 R", addInObj(pdf, pdf_doc, obj->getRef()));
+        break;
+    case objCmd:
+    case objError:
+    case objEOF:
+    case objNone:
+        pdftex_fail("PDF inclusion: type <%s> cannot be copied",
+                    obj->getTypeName());
+        break;
+    default:
+        assert(0);              // xpdf doesn't have any other types
+    }
+}
+
+//**********************************************************************
+
 static void writeRefs(PDF pdf, PdfDocument * pdf_doc)
 {
     InObj *r, *n;
+    XRef *xref;
+    xref = pdf_doc->doc->getXRef();
     for (r = pdf_doc->inObjList; r != NULL;) {
-        Object obj1;
-        pdf_doc->xref->fetch(r->ref.num, r->ref.gen, &obj1);
-        if (r->type == objFont) {
-            assert(!obj1.isStream());
+        PdfObject obj1;
+        xref->fetch(r->ref.num, r->ref.gen, &obj1);
+        if (obj1->isStream())
+            pdf_begin_obj(pdf, r->num, 0);
+        else
             pdf_begin_obj(pdf, r->num, 2);      // \pdfobjcompresslevel = 2 is for this
-            copyFontDict(pdf, pdf_doc, &obj1, r);
-            pdf_puts(pdf, "\n");
-            pdf_end_obj(pdf);
-        } else if (r->type != objFontDesc) {    // /FontDescriptor is written via write_fontdescriptor()
-            if (obj1.isStream())
-                pdf_begin_obj(pdf, r->num, 0);
-            else
-                pdf_begin_obj(pdf, r->num, 2);  // \pdfobjcompresslevel = 2 is for this
-            copyObject(pdf, pdf_doc, &obj1);
-            pdf_puts(pdf, "\n");
-            pdf_end_obj(pdf);
-        }
-        obj1.free();
+        copyObject(pdf, pdf_doc, &obj1);
+        pdf_puts(pdf, "\n");
+        pdf_end_obj(pdf);
         n = r->next;
         delete r;
         pdf_doc->inObjList = r = n;
     }
 }
 
-static void writeEncodings(PDF pdf)
-{
-    UsedEncoding *r, *n;
-    char *glyphNames[256], *s;
-    int i;
-    for (r = encodingList; r != NULL; r = r->next) {
-        for (i = 0; i < 256; i++) {
-            if (r->font->isCIDFont()) {
-                pdftex_fail
-                    ("PDF inclusion: CID fonts are not supported"
-                     " (try to disable font replacement to fix this)");
-            }
-            if ((s = ((Gfx8BitFont *) r->font)->getCharName(i)) != NULL)
-                glyphNames[i] = s;
-            else
-                glyphNames[i] = notdef;
-        }
-        epdf_write_enc(pdf, glyphNames, r->enc_objnum);
-    }
-    for (r = encodingList; r != NULL; r = n) {
-        n = r->next;
-        delete r->font;
-        delete r;
-    }
-}
-
 // get the pagebox coordinates according to the pagebox_spec
 
-static PDFRectangle *get_pagebox(Page * page, integer pagebox_spec)
+static PDFRectangle *get_pagebox(Page * page, int pagebox_spec)
 {
     switch (pagebox_spec) {
     case PDF_BOX_SPEC_MEDIA:
@@ -762,9 +463,8 @@ char *get_file_checksum(char *a)
 // Returns the page number.
 
 void
-read_pdf_info(PDF pdf,
-              image_dict * idict, integer minor_pdf_version_wanted,
-              integer pdf_inclusion_errorlevel, img_readtype_e readtype)
+read_pdf_info(image_dict * idict, int minor_pdf_version_wanted,
+              int pdf_inclusion_errorlevel, img_readtype_e readtype)
 {
     PdfDocument *pdf_doc;
     Page *page;
@@ -827,7 +527,6 @@ read_pdf_info(PDF pdf,
             pdftex_fail("PDF inclusion: required page <%i> does not exist",
                         (int) img_pagenum(idict));
     }
-    pdf_doc->xref = pdf_doc->doc->getXRef();
     // get the required page
     page = pdf_doc->doc->getCatalog()->getPage(img_pagenum(idict));
 
@@ -874,45 +573,29 @@ read_pdf_info(PDF pdf,
             ("PDF inclusion: "
              "/Rotate parameter in PDF file not multiple of 90 degrees.");
     }
-    if (page->getGroup() != NULL) {
-        initDictFromDict(pdf_doc, lastGroup, page->getGroup());
-        if (lastGroup->dictGetLength() > 0) {
-            groupIsIndirect = lastGroup->isRef();
-            if (groupIsIndirect) {
-                // FIXME: Here we already copy the object. It would be
-                // better to do this only after write_epdf, otherwise we
-                // may copy ununsed /Group objects
-                copyObject(pdf, pdf_doc, &lastGroup);
-                epdf_lastGroupObjectNum =
-                    getNewObjectNumber(pdf_doc, lastGroup->getRef());
-                pdf_puts(pdf, "\n");
-            } else {
-                // make the group an indirect object; copying is done later
-                // by write_additional_epdf_objects after write_epdf
-                epdf_lastGroupObjectNum = pdf_new_objnum(pdf);
-            }
-        }
-    } else {
-        epdf_lastGroupObjectNum = 0;
-    }
+
+    // currently unused info whether PDF contains a /Group
+    if (page->getGroup() != NULL)
+        img_set_group(idict);
+
     if (readtype == IMG_CLOSEINBETWEEN)
         unrefPdfDocument(img_filepath(idict));
 }
 
+//**********************************************************************
 // Writes the current epf_doc.
-// Here the included PDF is copied, so most errors that can happen during PDF
-// inclusion will arise here.
+// Here the included PDF is copied, so most errors that can happen
+// during PDF inclusion will arise here.
 
 static void write_epdf1(PDF pdf, image_dict * idict)
 {
+    PdfDocument *pdf_doc;
     Page *page;
-    PdfObject contents, obj1, obj2;
-    PdfObject metadata, pieceinfo, separationInfo;
-    Object info;
-    char *key, *checksum;
+    PdfObject info, contents, metadata, obj1;
+    Object *metadataNF, *resourcesNF;
+    char *checksum;
     char s[256];
     int i, l;
-    PdfDocument *pdf_doc;
     assert(idict != NULL);
     // calculate a checksum string
     checksum = get_file_checksum(img_filepath(idict));
@@ -923,9 +606,7 @@ static void write_epdf1(PDF pdf, image_dict * idict)
                     img_filename(idict));
     }
     free(checksum);
-    pdf_doc->xref = pdf_doc->doc->getXRef();
     (void) pdf_doc->doc->getCatalog()->getPage(img_pagenum(idict));
-    encodingList = NULL;
     page = pdf_doc->doc->getCatalog()->getPage(img_pagenum(idict));
     PDFRectangle *pagebox;
     float bbox[4];
@@ -942,10 +623,10 @@ static void write_epdf1(PDF pdf, image_dict * idict)
     pdf_printf(pdf, "/%s.PageNumber %i\n", pdfkeyprefix,
                (int) img_pagenum(idict));
     pdf_doc->doc->getDocInfoNF(&info);
-    if (info.isRef()) {
+    if (info->isRef()) {
         // the info dict must be indirect (PDF Ref p. 61)
         pdf_printf(pdf, "/%s.InfoDict ", pdfkeyprefix);
-        pdf_printf(pdf, "%d 0 R\n", addOther(info.getRef()));
+        pdf_printf(pdf, "%d 0 R\n", addInObj(pdf, pdf_doc, info->getRef()));
     }
     if (img_is_bbox(idict)) {
         bbox[0] = int2bp(img_bbox(idict)[0]);
@@ -965,73 +646,49 @@ static void write_epdf1(PDF pdf, image_dict * idict)
     pdf_puts(pdf, stripzeros(s));
     // The /Matrix calculation is replaced by transforms in out_img().
 
-    // write the page Group if it's there
-    if (epdf_lastGroupObjectNum > 0) {
-        if (page->getGroup() != NULL) {
-            initDictFromDict(pdf_doc, lastGroup, page->getGroup());
-            if (lastGroup->dictGetLength() > 0) {
-                pdf_puts(pdf, "/Group ");
-                groupIsIndirect = lastGroup->isRef();
-                pdf_printf(pdf, "%d 0 R", (int) epdf_lastGroupObjectNum);
-                pdf_puts(pdf, "\n");
-            }
-        }
-    }
-    // write the page Metadata if it's there
-    if (page->getMetadata() != NULL) {
-        metadata->initStream(page->getMetadata());
-        pdf_puts(pdf, "/Metadata ");
-        copyObject(pdf, pdf_doc, &metadata);
-        pdf_puts(pdf, "\n");
-    }
-    // write the page PieceInfo if it's there
-    if (page->getPieceInfo() != NULL) {
-        initDictFromDict(pdf_doc, pieceinfo, page->getPieceInfo());
-        if (pieceinfo->dictGetLength() > 0) {
-            pdf_puts(pdf, "/PieceInfo ");
-            copyObject(pdf, pdf_doc, &pieceinfo);
-            pdf_puts(pdf, "\n");
-        }
-    }
-    // copy LastModified (needed when PieceInfo is there)
-    if (page->getLastModified() != NULL) {
-        pdf_printf(pdf, "/LastModified (%s)\n",
-                   page->getLastModified()->getCString());
-    }
-    // write the page SeparationInfo if it's there
-    if (page->getSeparationInfo() != NULL) {
-        initDictFromDict(pdf_doc, separationInfo, page->getSeparationInfo());
-        if (separationInfo->dictGetLength() > 0) {
-            pdf_puts(pdf, "/SeparationInfo ");
-            copyObject(pdf, pdf_doc, &separationInfo);
-            pdf_puts(pdf, "\n");
-        }
-    }
-    // write the Resources dictionary
+    // Now all relevant parts of the /Page dictionary are copied:
+
+    // write the /Resources dictionary
     if (page->getResourceDict() == NULL) {
         // Resources can be missing (files without them have been spotted
         // in the wild); in which case the /Resouces of the /Page will be used.
         // "This practice is not recommended".
         pdftex_warn
-            ("PDF inclusion: /Resources missing. 'This practice is not recommended' (PDF Ref)");
+            ("PDF inclusion: /Resources missing. 'This practice is not recommended' (PDF Ref.)");
     } else {
-        initDictFromDict(pdf_doc, obj1, page->getResourceDict());
-        page->getResourceDict()->incRef();
-        if (!obj1->isDict())
-            pdftex_fail("PDF inclusion: invalid resources dict type <%s>",
-                        obj1->getTypeName());
-        pdf_puts(pdf, "/Resources <<\n");
-        for (i = 0, l = obj1->dictGetLength(); i < l; ++i) {
-            obj1->dictGetVal(i, &obj2);
-            key = obj1->dictGetKey(i);
-            if (strcmp("Font", key) == 0)
-                copyFontResources(pdf, pdf_doc, &obj2);
-            else if (strcmp("ProcSet", key) == 0)
-                copyProcSet(pdf, &obj2);
-            else
-                copyOtherResources(pdf, pdf_doc, &obj2, key);
+        resourcesNF = page->getResourcesNF();
+        pdf_puts(pdf, "/Resources ");
+        copyObject(pdf, pdf_doc, resourcesNF);
+    }
+    // write the page /Metadata if it's there (as a stream it must be indirect)
+    if (page->getMetadata() != NULL) {
+        metadataNF = page->getMetadataNF();     // *NF = "don't resolve indirects"
+        if (!metadataNF->isRef())
+            pdftex_warn("PDF inclusion: /Metadata must be indirect object");
+        else {
+            pdf_printf(pdf, "/Metadata %d 0 R",
+                       addInObj(pdf, pdf_doc, metadataNF->getRef()));
         }
-        pdf_puts(pdf, ">>\n");
+    }
+    // write the page /Group if it's there
+    if (page->getGroup() != NULL) {
+        pdf_puts(pdf, "/Group ");
+        copyDict(pdf, pdf_doc, page->getGroup());
+    }
+    // write the page /PieceInfo if it's there
+    if (page->getPieceInfo() != NULL) {
+        pdf_puts(pdf, "/PieceInfo ");
+        copyDict(pdf, pdf_doc, page->getPieceInfo());
+    }
+    // write the page /SeparationInfo if it's there
+    if (page->getSeparationInfo() != NULL) {
+        pdf_puts(pdf, "/SeparationInfo ");
+        copyDict(pdf, pdf_doc, page->getSeparationInfo());
+    }
+    // copy LastModified (needed when PieceInfo is there)
+    if (page->getLastModified() != NULL) {
+        pdf_printf(pdf, "/LastModified (%s)\n",
+                   page->getLastModified()->getCString());
     }
     // write the page contents
     page->getContents(&contents);
@@ -1040,7 +697,7 @@ static void write_epdf1(PDF pdf, image_dict * idict)
         // of \pdfcompresslevel
         //
         // pdfbeginstream();
-        // copyStream(contents->getStream());
+        // copyStreamStream(contents->getStream());
         // pdfendstream();
 
         // Variant B: copy stream without recompressing
@@ -1067,14 +724,20 @@ static void write_epdf1(PDF pdf, image_dict * idict)
             }
         }
         pdf_puts(pdf, ">>\nstream\n");
-        copyStream(pdf, contents->getStream()->getBaseStream());
+        copyStreamStream(pdf, contents->getStream()->getBaseStream());
         pdf_end_stream(pdf);
     } else if (contents->isArray()) {
         pdf_begin_stream(pdf);
         for (i = 0, l = contents->arrayGetLength(); i < l; ++i) {
-            Object contentsobj;
-            copyStream(pdf, (contents->arrayGet(i, &contentsobj))->getStream());
-            contentsobj.free();
+            PdfObject contentsobj;
+            copyStreamStream(pdf,
+                             (contents->arrayGet(i, &contentsobj))->
+                             getStream());
+            if (i < (l - 1)) {
+                // put a space between streams to be on the safe side (streams
+                // should have a trailing space here, but one never knows)
+                pdf_puts(pdf, " ");
+            }
         }
         pdf_end_stream(pdf);
     } else {                    // the contents are optional, but we need to include an empty stream
@@ -1083,8 +746,6 @@ static void write_epdf1(PDF pdf, image_dict * idict)
     }
     // write out all indirect objects
     writeRefs(pdf, pdf_doc);
-    // write out all used encodings (and delete list)
-    writeEncodings(pdf);
 }
 
 // this relay function is needed to keep some destructor quiet (???)
@@ -1152,16 +813,3 @@ void epdf_check_mem()
         delete globalParams;
     isInit = gFalse;
 }
-
-// Called after the xobject generated by write_epdf has been finished; used to
-// write out objects that have been made indirect
-
-void write_additional_epdf_objects(PDF pdf, char *file_path)
-{
-    PdfDocument *pdf_doc = findPdfDocument(file_path);
-    if ((epdf_lastGroupObjectNum > 0) && !groupIsIndirect) {
-        pdf_begin_obj(pdf, epdf_lastGroupObjectNum, 2);
-        copyObject(pdf, pdf_doc, &lastGroup);
-        pdf_end_obj(pdf);
-    }
-}
