@@ -18,13 +18,10 @@
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
 #include "lua/luatex-api.h"
-#include <ptexlib.h>
+#include "ptexlib.h"
 
 static const char _svn_version[] =
-    "$Id: lcallbacklib.c 3261 2009-12-18 11:38:21Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.50.0/source/texk/web2c/luatexdir/lua/lcallbacklib.c $";
-
-extern int do_run_callback(int special, char *values, va_list vl);
-extern int lua_traceback(lua_State * L);
+    "$Id: lcallbacklib.c 3404 2010-01-28 11:17:10Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.60.0/source/texk/web2c/luatexdir/lua/lcallbacklib.c $";
 
 int callback_count = 0;
 int saved_callback_count = 0;
@@ -71,7 +68,7 @@ static const char *const callbacknames[] = {
 
 int callback_callbacks_id = 0;
 
-void get_lua_boolean(char *table, char *name, boolean * target)
+void get_lua_boolean(const char *table, const char *name, boolean * target)
 {
     int stacktop;
     stacktop = lua_gettop(Luas);
@@ -80,16 +77,16 @@ void get_lua_boolean(char *table, char *name, boolean * target)
     if (lua_istable(Luas, -1)) {
         lua_getfield(Luas, -1, name);
         if (lua_isboolean(Luas, -1)) {
-            *target = (lua_toboolean(Luas, -1));
+            *target = (boolean) (lua_toboolean(Luas, -1));
         } else if (lua_isnumber(Luas, -1)) {
-            *target = (lua_tonumber(Luas, -1) == 0 ? 0 : 1);
+            *target = (boolean) (lua_tonumber(Luas, -1) == 0 ? 0 : 1);
         }
     }
     lua_settop(Luas, stacktop);
     return;
 }
 
-void get_saved_lua_boolean(int r, char *name, boolean * target)
+void get_saved_lua_boolean(int r, const char *name, boolean * target)
 {
     int stacktop;
     stacktop = lua_gettop(Luas);
@@ -98,16 +95,16 @@ void get_saved_lua_boolean(int r, char *name, boolean * target)
     if (lua_istable(Luas, -1)) {
         lua_getfield(Luas, -1, name);
         if (lua_isboolean(Luas, -1)) {
-            *target = lua_toboolean(Luas, -1);
+            *target = (boolean) lua_toboolean(Luas, -1);
         } else if (lua_isnumber(Luas, -1)) {
-            *target = (lua_tonumber(Luas, -1) == 0 ? 0 : 1);
+            *target = (boolean) (lua_tonumber(Luas, -1) == 0 ? 0 : 1);
         }
     }
     lua_settop(Luas, stacktop);
     return;
 }
 
-void get_lua_number(char *table, char *name, int * target)
+void get_lua_number(const char *table, const char *name, int *target)
 {
     int stacktop;
     stacktop = lua_gettop(Luas);
@@ -116,14 +113,14 @@ void get_lua_number(char *table, char *name, int * target)
     if (lua_istable(Luas, -1)) {
         lua_getfield(Luas, -1, name);
         if (lua_isnumber(Luas, -1)) {
-            *target = lua_tonumber(Luas, -1);
+            lua_number2int(*target, lua_tonumber(Luas, -1));
         }
     }
     lua_settop(Luas, stacktop);
     return;
 }
 
-void get_saved_lua_number(int r, char *name, int * target)
+void get_saved_lua_number(int r, const char *name, int *target)
 {
     int stacktop;
     stacktop = lua_gettop(Luas);
@@ -132,7 +129,7 @@ void get_saved_lua_number(int r, char *name, int * target)
     if (lua_istable(Luas, -1)) {
         lua_getfield(Luas, -1, name);
         if (lua_isnumber(Luas, -1)) {
-            *target = lua_tonumber(Luas, -1);
+            lua_number2int(*target, lua_tonumber(Luas, -1));
         }
     }
     lua_settop(Luas, stacktop);
@@ -140,7 +137,7 @@ void get_saved_lua_number(int r, char *name, int * target)
 }
 
 
-void get_lua_string(char *table, char *name, char **target)
+void get_lua_string(const char *table, const char *name, char **target)
 {
     int stacktop;
     stacktop = lua_gettop(Luas);
@@ -149,14 +146,14 @@ void get_lua_string(char *table, char *name, char **target)
     if (lua_istable(Luas, -1)) {
         lua_getfield(Luas, -1, name);
         if (lua_isstring(Luas, -1)) {
-            *target = (char *) lua_tostring(Luas, -1);
+            *target = xstrdup(lua_tostring(Luas, -1));
         }
     }
     lua_settop(Luas, stacktop);
     return;
 }
 
-void get_saved_lua_string(int r, char *name, char **target)
+void get_saved_lua_string(int r, const char *name, char **target)
 {
     int stacktop;
     stacktop = lua_gettop(Luas);
@@ -165,7 +162,7 @@ void get_saved_lua_string(int r, char *name, char **target)
     if (lua_istable(Luas, -1)) {
         lua_getfield(Luas, -1, name);
         if (lua_isstring(Luas, -1)) {
-            *target = (char *) lua_tostring(Luas, -1);
+            *target = xstrdup(lua_tostring(Luas, -1));
         }
     }
     lua_settop(Luas, stacktop);
@@ -181,7 +178,7 @@ void get_saved_lua_string(int r, char *name, char **target)
 #define CALLBACK_CHARNUM        'c'
 
 
-int run_saved_callback(int r, char *name, char *values, ...)
+int run_saved_callback(int r, const char *name, const char *values, ...)
 {
     va_list args;
     int ret = 0;
@@ -216,7 +213,7 @@ boolean get_callback(lua_State * L, int i)
     }
 }
 
-int run_and_save_callback(int i, char *values, ...)
+int run_and_save_callback(int i, const char *values, ...)
 {
     va_list args;
     int ret = 0;
@@ -235,7 +232,7 @@ int run_and_save_callback(int i, char *values, ...)
 }
 
 
-int run_callback(int i, char *values, ...)
+int run_callback(int i, const char *values, ...)
 {
     va_list args;
     int ret = 0;
@@ -250,7 +247,7 @@ int run_callback(int i, char *values, ...)
     return ret;
 }
 
-int do_run_callback(int special, char *values, va_list vl)
+int do_run_callback(int special, const char *values, va_list vl)
 {
     int ret;
     size_t len;
@@ -267,7 +264,7 @@ int do_run_callback(int special, char *values, va_list vl)
     }
     ss = index(values, '>');
     assert(ss);
-    luaL_checkstack(L, (ss - values + 1), "out of stack space");
+    luaL_checkstack(L, (int) (ss - values + 1), "out of stack space");
     ss = NULL;
     for (narg = 0; *values; narg++) {
         switch (*values++) {
@@ -290,7 +287,8 @@ int do_run_callback(int special, char *values, va_list vl)
             lua_pushboolean(L, va_arg(vl, int));
             break;
         case CALLBACK_LINE:    /* a buffer section, with implied start */
-            lua_pushlstring(L, (char *) (buffer + first), va_arg(vl, int));
+            lua_pushlstring(L, (char *) (buffer + first),
+                            (size_t) va_arg(vl, int));
             break;
         case '-':
             narg--;
@@ -302,7 +300,7 @@ int do_run_callback(int special, char *values, va_list vl)
         }
     }
   ENDARGS:
-    nres = strlen(values);
+    nres = (int) strlen(values);
     if (special == 1) {
         nres++;
     }
@@ -351,7 +349,7 @@ int do_run_callback(int special, char *values, va_list vl)
                         lua_typename(L, lua_type(L, nres)));
                 goto EXIT;
             }
-            b = lua_tonumber(L, nres);
+            lua_number2int(b, lua_tonumber(L, nres));
             *va_arg(vl, int *) = b;
             break;
         case CALLBACK_LINE:    /* TeX line */
@@ -366,9 +364,9 @@ int do_run_callback(int special, char *values, va_list vl)
                 bufloc = va_arg(vl, int *);
                 if (len != 0) {
                     ret = *bufloc;
-                    check_buffer_overflow(ret + len);
+                    check_buffer_overflow(ret + (int) len);
                     strncpy((char *) (buffer + ret), s, len);
-                    *bufloc += len;
+                    *bufloc += (int) len;
                     /* while (len--) {  buffer[(*bufloc)++] = *s++; } */
                     while ((*bufloc) - 1 > ret && buffer[(*bufloc) - 1] == ' ')
                         (*bufloc)--;
@@ -405,7 +403,7 @@ int do_run_callback(int special, char *values, va_list vl)
             if (s == NULL)      /* |len| can be zero */
                 *va_arg(vl, int *) = 0;
             else {
-                ss = xmalloc(len + 1);
+                ss = xmalloc((unsigned) (len + 1));
                 (void) memcpy(ss, s, (len + 1));
                 *va_arg(vl, char **) = ss;
             }
@@ -429,7 +427,7 @@ void destroy_saved_callback(int i)
 static int callback_register(lua_State * L)
 {
     int cb;
-    char *s;
+    const char *s;
     if (!lua_isstring(L, 1) ||
         ((!lua_isfunction(L, 2)) &&
          (!lua_isnil(L, 2)) &&
@@ -438,7 +436,7 @@ static int callback_register(lua_State * L)
         lua_pushstring(L, "Invalid arguments to callback.register.");
         return 2;
     }
-    s = (char *) lua_tostring(L, 1);
+    s = lua_tostring(L, 1);
     for (cb = 0; cb < total_callbacks; cb++) {
         if (strcmp(callbacknames[cb], s) == 0)
             break;
@@ -467,13 +465,13 @@ static int callback_register(lua_State * L)
 static int callback_find(lua_State * L)
 {
     int cb;
-    char *s;
+    const char *s;
     if (!lua_isstring(L, 1)) {
         lua_pushnil(L);
         lua_pushstring(L, "Invalid arguments to callback.find.");
         return 2;
     }
-    s = (char *) lua_tostring(L, 1);
+    s = lua_tostring(L, 1);
     for (cb = 0; cb < total_callbacks; cb++) {
         if (strcmp(callbacknames[cb], s) == 0)
             break;
