@@ -1,6 +1,6 @@
 /* lpdflib.c
 
-   Copyright 2006-2009 Taco Hoekwater <taco@luatex.org>
+   Copyright 2006-2010 Taco Hoekwater <taco@luatex.org>
 
    This file is part of LuaTeX.
 
@@ -18,53 +18,47 @@
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
 static const char _svn_version[] =
-    "$Id: lpdflib.c 3248 2009-12-11 13:39:41Z taco $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/lua/lpdflib.c $";
+    "$Id: lpdflib.c 3582 2010-04-02 16:37:08Z hhenkel $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.60.0/source/texk/web2c/luatexdir/lua/lpdflib.c $";
 
 #include "lua/luatex-api.h"
-#include <ptexlib.h>
+#include "ptexlib.h"
 
 #define buf_to_pdfbuf_macro(p, s, l)              \
 for (i = 0; i < (l); i++) {                       \
     if (i % 16 == 0)                              \
         pdf_room(p, 16);                          \
-    pdf_quick_out(p, ((unsigned char *) (s))[i]); \
+    pdf_quick_out(p, ((const unsigned char *) (s))[i]); \
 }
 
 int luapdfprint(lua_State * L)
 {
     int n;
     unsigned i;
-    lstring st, modestr;
+    const_lstring st, modestr;
     ctm_transform_modes literal_mode;
     st.s = modestr.s = NULL;
     n = lua_gettop(L);
     if (!lua_isstring(L, -1)) {
-        lua_pushstring(L, "no string to print");
-        lua_error(L);
+        luaL_error(L, "no string to print");
     }
     literal_mode = set_origin;
     if (n == 2) {
         if (!lua_isstring(L, -2)) {
-            lua_pushstring(L, "invalid argument for print literal mode");
-            lua_error(L);
+            luaL_error(L, "invalid argument for print literal mode");
         } else {
-            modestr.s = (unsigned char *) lua_tolstring(L, -2, &modestr.l);
-            if (modestr.l == 6
-                && strncmp((const char *) modestr.s, "direct", 6) == 0)
+            modestr.s = lua_tolstring(L, -2, &modestr.l);
+            if (modestr.l == 6 && strncmp(modestr.s, "direct", 6) == 0)
                 literal_mode = direct_always;
-            else if (modestr.l == 4
-                     && strncmp((const char *) modestr.s, "page", 4) == 0)
+            else if (modestr.l == 4 && strncmp(modestr.s, "page", 4) == 0)
                 literal_mode = direct_page;
             else {
-                lua_pushstring(L, "invalid argument for print literal mode");
-                lua_error(L);
+                luaL_error(L, "invalid argument for print literal mode");
             }
         }
     } else {
         if (n != 1) {
-            lua_pushstring(L, "invalid number of arguments");
-            lua_error(L);
+            luaL_error(L, "invalid number of arguments");
         }
     }
     check_o_mode(static_pdf, "pdf.print()", 1 << OMODE_PDF, true);
@@ -84,21 +78,23 @@ int luapdfprint(lua_State * L)
     default:
         assert(0);
     }
-    st.s = (unsigned char *) lua_tolstring(L, n, &st.l);
+    st.s = lua_tolstring(L, n, &st.l);
     buf_to_pdfbuf_macro(static_pdf, st.s, st.l);
     return 0;
 }
 
-unsigned char *fread_to_buf(lua_State * L, char *filename, size_t * len)
+unsigned char *fread_to_buf(lua_State * L, const char *filename, size_t * len)
 {
     int i = 0;
+    int ilen = 0;
     FILE *f;
     unsigned char *buf = NULL;
     if ((f = fopen(filename, "rb")) == NULL)
         luaL_error(L, "pdf.immediateobj() cannot open input file");
-    if ((i = readbinfile(f, &buf, (integer *) len)) == 0)
+    if ((i = readbinfile(f, &buf, &ilen)) == 0)
         luaL_error(L, "pdf.immediateobj() cannot read input file");
     fclose(f);
+    *len = (size_t) ilen;
     return buf;
 }
 
@@ -106,14 +102,15 @@ static int l_immediateobj(lua_State * L)
 {
     int n, first_arg = 1;
     unsigned i;
-    integer k;
-    lstring buf, st1, st2, st3;
-    buf.s = st1.s = st2.s = st3.s = NULL;
+    int k;
+    lstring buf;
+    const_lstring st1, st2, st3;
+    st1.s = st2.s = st3.s = NULL;
     check_o_mode(static_pdf, "immediateobj()", 1 << OMODE_PDF, true);
     n = lua_gettop(L);
     if (n > 0 && lua_type(L, 1) == LUA_TNUMBER) {
         first_arg++;
-        k = lua_tonumber(L, 1);
+        lua_number2int(k, lua_tonumber(L, 1));
         check_obj_exists(static_pdf, obj_type_obj, k);
         if (is_obj_scheduled(static_pdf, k) || obj_data_ptr(static_pdf, k) != 0)
             luaL_error(L, "pdf.immediateobj() object in use");
@@ -131,7 +128,7 @@ static int l_immediateobj(lua_State * L)
         if (!lua_isstring(L, first_arg))
             luaL_error(L, "pdf.immediateobj() 1st argument must be string");
         pdf_begin_obj(static_pdf, k, 1);
-        st1.s = (unsigned char *) lua_tolstring(L, first_arg, &st1.l);
+        st1.s = lua_tolstring(L, first_arg, &st1.l);
         buf_to_pdfbuf_macro(static_pdf, st1.s, st1.l);
         if (st1.s[st1.l - 1] != '\n')
             pdf_puts(static_pdf, "\n");
@@ -143,14 +140,14 @@ static int l_immediateobj(lua_State * L)
             luaL_error(L, "pdf.immediateobj() 1st argument must be string");
         if (!lua_isstring(L, first_arg + 1))
             luaL_error(L, "pdf.immediateobj() 2nd argument must be string");
-        st1.s = (unsigned char *) lua_tolstring(L, first_arg, &st1.l);
-        st2.s = (unsigned char *) lua_tolstring(L, first_arg + 1, &st2.l);
+        st1.s = lua_tolstring(L, first_arg, &st1.l);
+        st2.s = lua_tolstring(L, first_arg + 1, &st2.l);
         if (st1.l == 4 && strncmp((const char *) st1.s, "file", 4) == 0) {
             if (n == first_arg + 2)
                 luaL_error(L,
                            "pdf.immediateobj() 3rd argument forbidden in file mode");
             pdf_begin_obj(static_pdf, k, 1);
-            buf.s = fread_to_buf(L, (char *) st2.s, &buf.l);
+            buf.s = fread_to_buf(L, st2.s, &buf.l);
             buf_to_pdfbuf_macro(static_pdf, buf.s, buf.l);
             if (buf.s[buf.l - 1] != '\n')
                 pdf_puts(static_pdf, "\n");
@@ -162,7 +159,7 @@ static int l_immediateobj(lua_State * L)
                 if (!lua_isstring(L, first_arg + 2))
                     luaL_error(L,
                                "pdf.immediateobj() 3rd argument must be string");
-                st3.s = (unsigned char *) lua_tolstring(L, first_arg + 2, &st3.l);
+                st3.s = lua_tolstring(L, first_arg + 2, &st3.l);
                 buf_to_pdfbuf_macro(static_pdf, st3.s, st3.l);
                 if (st3.s[st3.l - 1] != '\n')
                     pdf_puts(static_pdf, "\n");
@@ -173,7 +170,7 @@ static int l_immediateobj(lua_State * L)
             } else if (st1.l == 10
                        && strncmp((const char *) st1.s, "streamfile",
                                   10) == 0) {
-                buf.s = fread_to_buf(L, (char *) st2.s, &buf.l);
+                buf.s = fread_to_buf(L, st2.s, &buf.l);
                 buf_to_pdfbuf_macro(static_pdf, buf.s, buf.l);
                 xfree(buf.s);
             } else
@@ -191,17 +188,42 @@ static int l_immediateobj(lua_State * L)
 /**********************************************************************/
 /* for LUA_ENVIRONINDEX table lookup (instead of repeated strcmp()) */
 
-typedef enum { P__ZERO, P_RAW, P_STREAM, P__SENTINEL } parm_idx;
+typedef enum { P__ZERO,
+    P_CATALOG,
+    P_H,
+    P_INFO,
+    P_NAMES,
+    P_PAGEATTRIBUTES,
+    P_PAGERESOURCES,
+    P_PAGESATTRIBUTES,
+    P_PDFCATALOG,
+    P_PDFINFO,
+    P_PDFNAMES,
+    P_PDFTRAILER,
+    P_RAW,
+    P_STREAM,
+    P_TRAILER,
+    P_V,
+    P__SENTINEL
+} parm_idx;
 
-typedef struct {
-    const char *name;           /* parameter name */
-    parm_idx idx;               /* index within img_parms array */
-} parm_struct;
-
-const parm_struct obj_parms[] = {
+static const parm_struct pdf_parms[] = {
     {NULL, P__ZERO},            /* dummy; lua indices run from 1 */
+    {"catalog", P_CATALOG},
+    {"h", P_H},
+    {"info", P_INFO},
+    {"names", P_NAMES},
+    {"pageattributes", P_PAGEATTRIBUTES},
+    {"pageresources", P_PAGERESOURCES},
+    {"pagesattributes", P_PAGESATTRIBUTES},
+    {"pdfcatalog", P_PDFCATALOG},       /* obsolescent */
+    {"pdfinfo", P_PDFINFO},     /* obsolescent */
+    {"pdfnames", P_PDFNAMES},   /* obsolescent */
+    {"pdftrailer", P_PDFTRAILER},       /* obsolescent */
     {"raw", P_RAW},
     {"stream", P_STREAM},
+    {"trailer", P_TRAILER},
+    {"v", P_V},
     {NULL, P__SENTINEL}
 };
 
@@ -213,10 +235,12 @@ static int table_obj(lua_State * L)
     int compress_level = -1;    /* unset */
     int os_level = 1;           /* default: put non-stream objects into object streams */
     int saved_compress_level = static_pdf->compress_level;
-    lstring attr, st, buf;
+    const_lstring attr, st;
+    lstring buf;
     size_t i;
     int immediate = 0;          /* default: not immediate */
-    attr.s = st.s = buf.s = NULL;
+    attr.s = st.s = NULL;
+    attr.l = 0;
     assert(lua_istable(L, 1));  /* t */
 
     /* get object "type" */
@@ -232,16 +256,15 @@ static int table_obj(lua_State * L)
     if (!lua_isnumber(L, -1))   /* !i vs t */
         luaL_error(L, "pdf.obj(): \"%s\" is not a valid object type",
                    lua_tostring(L, -2));
-    type = lua_tointeger(L, -1);        /* i vs t */
-    lua_pop(L, 1);              /* vs t */
+    type = (int) lua_tointeger(L, -1);  /* i vs t */
     switch (type) {
     case P_RAW:
     case P_STREAM:
         break;
     default:
-        assert(0);
+        luaL_error(L, "pdf.obj(): \"%s\" is not a valid object type", lua_tostring(L, -2));     /* i vs t */
     }
-    lua_pop(L, 1);              /* t */
+    lua_pop(L, 2);              /* t */
 
     /* get optional "immediate" */
 
@@ -261,7 +284,7 @@ static int table_obj(lua_State * L)
     if (!lua_isnil(L, -1)) {    /* vi? t */
         if (!lua_isnumber(L, -1))       /* !vi t */
             luaL_error(L, "pdf.obj(): \"objnum\" must be integer");
-        k = lua_tointeger(L, -1);       /* vi t */
+        k = (int) lua_tointeger(L, -1); /* vi t */
         check_obj_exists(static_pdf, obj_type_obj, k);
         if (is_obj_scheduled(static_pdf, k) || obj_data_ptr(static_pdf, k) != 0)
             luaL_error(L, "pdf.obj() object in use");
@@ -288,7 +311,7 @@ static int table_obj(lua_State * L)
         if (!lua_isstring(L, -1))       /* !attr-s t */
             luaL_error(L, "pdf.obj(): object \"attr\" must be string");
         if (immediate == 1) {
-            attr.s = (unsigned char *) lua_tolstring(L, -1, &attr.l);    /* attr-s t */
+            attr.s = lua_tolstring(L, -1, &attr.l);     /* attr-s t */
             lua_pop(L, 1);      /* t */
         } else
             obj_obj_stream_attr(static_pdf, k) = luaL_ref(Luas, LUA_REGISTRYINDEX);     /* t */
@@ -305,7 +328,7 @@ static int table_obj(lua_State * L)
                        "pdf.obj(): \"compresslevel\" key not allowed for raw object");
         if (!lua_isnumber(L, -1))       /* !vi t */
             luaL_error(L, "pdf.obj(): \"compresslevel\" must be integer");
-        compress_level = lua_tointeger(L, -1);  /* vi t */
+        compress_level = (int) lua_tointeger(L, -1);    /* vi t */
         if (compress_level > 9)
             luaL_error(L, "pdf.obj(): \"compresslevel\" must be <= 9");
         else if (compress_level < 0)
@@ -355,7 +378,7 @@ static int table_obj(lua_State * L)
                 luaL_error(L,
                            "pdf.obj(): \"string\" must be string for raw object");
             if (immediate == 1) {
-                st.s = (unsigned char *) lua_tolstring(L, -1, &st.l);
+                st.s = lua_tolstring(L, -1, &st.l);
                 buf_to_pdfbuf_macro(static_pdf, st.s, st.l);
                 if (st.s[st.l - 1] != '\n')
                     pdf_puts(static_pdf, "\n");
@@ -367,8 +390,8 @@ static int table_obj(lua_State * L)
                 luaL_error(L,
                            "pdf.obj(): \"file\" name must be string for raw object");
             if (immediate == 1) {
-                st.s = (unsigned char *) lua_tolstring(L, -1, &st.l);    /* file-s nil t */
-                buf.s = fread_to_buf(L, (char *) st.s, &buf.l);
+                st.s = lua_tolstring(L, -1, &st.l);     /* file-s nil t */
+                buf.s = fread_to_buf(L, st.s, &buf.l);
                 buf_to_pdfbuf_macro(static_pdf, buf.s, buf.l);
                 if (buf.s[buf.l - 1] != '\n')
                     pdf_puts(static_pdf, "\n");
@@ -404,7 +427,7 @@ static int table_obj(lua_State * L)
                 luaL_error(L,
                            "pdf.obj(): \"string\" must be string for stream object");
             if (immediate == 1) {
-                st.s = (unsigned char *) lua_tolstring(L, -1, &st.l);    /* string-s t */
+                st.s = lua_tolstring(L, -1, &st.l);     /* string-s t */
                 buf_to_pdfbuf_macro(static_pdf, st.s, st.l);
             } else
                 obj_obj_data(static_pdf, k) = luaL_ref(L, LUA_REGISTRYINDEX);   /* t */
@@ -414,8 +437,8 @@ static int table_obj(lua_State * L)
                 luaL_error(L,
                            "pdf.obj(): \"file\" name must be string for stream object");
             if (immediate == 1) {
-                st.s = (unsigned char *) lua_tolstring(L, -1, &st.l);    /* file-s nil t */
-                buf.s = fread_to_buf(L, (char *) st.s, &buf.l);
+                st.s = lua_tolstring(L, -1, &st.l);     /* file-s nil t */
+                buf.s = fread_to_buf(L, st.s, &buf.l);
                 buf_to_pdfbuf_macro(static_pdf, buf.s, buf.l);
                 xfree(buf.s);
             } else {
@@ -436,13 +459,13 @@ static int table_obj(lua_State * L)
 static int orig_obj(lua_State * L)
 {
     int n, first_arg = 1;
-    integer k;
-    lstring st;
+    int k;
+    const_lstring st;
     st.s = NULL;
     n = lua_gettop(L);
     if (n > 0 && lua_type(L, 1) == LUA_TNUMBER) {
         first_arg++;
-        k = lua_tonumber(L, 1);
+        lua_number2int(k, lua_tonumber(L, 1));
         check_obj_exists(static_pdf, obj_type_obj, k);
         if (is_obj_scheduled(static_pdf, k) || obj_data_ptr(static_pdf, k) != 0)
             luaL_error(L, "pdf.obj() object in use");
@@ -468,7 +491,7 @@ static int orig_obj(lua_State * L)
             luaL_error(L, "pdf.obj() 1st argument must be string");
         if (!lua_isstring(L, first_arg + 1))
             luaL_error(L, "pdf.obj() 2nd argument must be string");
-        st.s = (unsigned char *) lua_tolstring(L, first_arg, &st.l);
+        st.s = lua_tolstring(L, first_arg, &st.l);
         if (st.l == 4 && strncmp((const char *) st.s, "file", 4) == 0) {
             if (n == first_arg + 2)
                 luaL_error(L, "pdf.obj() 3rd argument forbidden in file mode");
@@ -500,7 +523,8 @@ static int orig_obj(lua_State * L)
 static int l_obj(lua_State * L)
 {
     int n;
-    integer k;
+    int k;
+    ensure_output_state(static_pdf, ST_HEADER_WRITTEN);
     n = lua_gettop(L);
     if (n == 1 && lua_istable(L, 1))
         k = table_obj(L);       /* new */
@@ -513,7 +537,7 @@ static int l_obj(lua_State * L)
 static int l_reserveobj(lua_State * L)
 {
     int n;
-    lstring st;
+    const_lstring st;
     st.s = 0;
     n = lua_gettop(L);
     switch (n) {
@@ -525,7 +549,7 @@ static int l_reserveobj(lua_State * L)
     case 1:
         if (!lua_isstring(L, -1))
             luaL_error(L, "pdf.reserveobj() optional argument must be string");
-        st.s = (unsigned char *) lua_tolstring(L, 1, &st.l);
+        st.s = lua_tolstring(L, 1, &st.l);
         if (st.l == 5 && strncmp((const char *) st.s, "annot", 5) == 0) {
             pdf_create_obj(static_pdf, obj_type_annot, 0);
             pdf_last_annot = static_pdf->sys_obj_ptr;
@@ -549,11 +573,11 @@ static int l_registerannot(lua_State * L)
     case 1:
         if (!is_shipping_page)
             luaL_error(L, "pdf.registerannot() can only be used in late lua");
-        i = luaL_checkinteger(L, 1);
+        i = (int) luaL_checkinteger(L, 1);
         if (i <= 0)
             luaL_error(L,
                        "pdf.registerannot() can only register positive object numbers");
-        append_object_list(static_pdf, obj_type_annot, (-i));
+        addto_page_resources(static_pdf, obj_type_annot, -i);
         break;
     default:
         luaL_error(L, "pdf.registerannot() needs exactly 1 argument");
@@ -563,32 +587,60 @@ static int l_registerannot(lua_State * L)
 
 static int getpdf(lua_State * L)
 {
-    char *st, *s;
-    int l;
-    if (lua_isstring(L, 2)) {
-        st = (char *) lua_tostring(L, 2);
-        if (st) {
-            if (strcmp(st, "h") == 0) {
-                lua_pushnumber(L, static_pdf->posstruct->pos.h);
-            } else if (strcmp(st, "v") == 0) {
-                lua_pushnumber(L, static_pdf->posstruct->pos.v);
-            } else if (strcmp(st, "pdfcatalog") == 0) {
+    char *s;
+    const char *st;
+    int i, l;
+    if (lua_isstring(L, 2) && (st = lua_tostring(L, 2)) != NULL) {
+        lua_pushvalue(L, 2);    /* st ... */
+        lua_gettable(L, LUA_ENVIRONINDEX);      /* i? ... */
+        if (lua_isnumber(L, -1)) {      /* i ... */
+            i = (int) lua_tointeger(L, -1);     /* i ... */
+            lua_pop(L, 1);      /* ... */
+            switch (i) {
+            case P_PAGEATTRIBUTES:
+                s = tokenlist_to_cstring(pdf_pageattributes_toks, true, &l);
+                lua_pushlstring(L, s, (size_t) l);
+                break;
+            case P_PAGERESOURCES:
+                s = tokenlist_to_cstring(pdf_pageresources_toks, true, &l);
+                lua_pushlstring(L, s, (size_t) l);
+                break;
+            case P_PAGESATTRIBUTES:
+                s = tokenlist_to_cstring(pdf_pagesattributes_toks, true, &l);
+                lua_pushlstring(L, s, (size_t) l);
+                break;
+            case P_PDFCATALOG:
+            case P_CATALOG:
                 s = tokenlist_to_cstring(pdf_catalog_toks, true, &l);
-                lua_pushlstring(L, s, l);
-            } else if (strcmp(st, "pdfinfo") == 0) {
+                lua_pushlstring(L, s, (size_t) l);
+                break;
+            case P_PDFINFO:
+            case P_INFO:
                 s = tokenlist_to_cstring(pdf_info_toks, true, &l);
-                lua_pushlstring(L, s, l);
-            } else if (strcmp(st, "pdfnames") == 0) {
+                lua_pushlstring(L, s, (size_t) l);
+                break;
+            case P_PDFNAMES:
+            case P_NAMES:
                 s = tokenlist_to_cstring(pdf_names_toks, true, &l);
-                lua_pushlstring(L, s, l);
-            } else if (strcmp(st, "pdftrailer") == 0) {
+                lua_pushlstring(L, s, (size_t) l);
+                break;
+            case P_PDFTRAILER:
+            case P_TRAILER:
                 s = tokenlist_to_cstring(pdf_trailer_toks, true, &l);
-                lua_pushlstring(L, s, l);
-            } else {
-                lua_pushnil(L);
+                lua_pushlstring(L, s, (size_t) l);
+                break;
+            case P_H:
+                lua_pushnumber(L, static_pdf->posstruct->pos.h);
+                break;
+            case P_V:
+                lua_pushnumber(L, static_pdf->posstruct->pos.v);
+                break;
+            default:
+                lua_rawget(L, -2);
             }
         } else {
-            lua_pushnil(L);
+            lua_pop(L, 1);      /* ... */
+            lua_rawget(L, -2);
         }
     } else {
         lua_pushnil(L);
@@ -598,34 +650,99 @@ static int getpdf(lua_State * L)
 
 static int setpdf(lua_State * L)
 {
-    char *st;
+    int i;
     if (lua_gettop(L) != 3) {
         return 0;
     }
-    /* can't set |h| and |v| yet */
-    st = (char *) luaL_checkstring(L, 2);
-    if (strcmp(st, "pdfcatalog") == 0) {
-        pdf_catalog_toks = tokenlist_from_lua(L);
-    } else if (strcmp(st, "pdfinfo") == 0) {
-        pdf_info_toks = tokenlist_from_lua(L);
-    } else if (strcmp(st, "pdfnames") == 0) {
-        pdf_names_toks = tokenlist_from_lua(L);
-    } else if (strcmp(st, "pdftrailer") == 0) {
-        pdf_trailer_toks = tokenlist_from_lua(L);
-    } else if (strcmp(st, "pdfmapline") == 0) {
-        char *s = (char *) lua_tostring(L, -1);
-        process_map_item(s, MAPLINE);
-    } else if (strcmp(st, "pdfmapfile") == 0) {
-        char *s = (char *) lua_tostring(L, -1);
-        process_map_item(s, MAPFILE);
+    (void) luaL_checkstring(L, 2);      /* ... */
+    lua_pushvalue(L, 2);        /* st ... */
+    lua_gettable(L, LUA_ENVIRONINDEX);  /* i? ... */
+    if (lua_isnumber(L, -1)) {  /* i ... */
+        i = (int) lua_tointeger(L, -1); /* i ... */
+        lua_pop(L, 1);          /* ... */
+        switch (i) {
+        case P_PAGEATTRIBUTES:
+            pdf_pageattributes_toks = tokenlist_from_lua(L);
+            break;
+        case P_PAGERESOURCES:
+            pdf_pageresources_toks = tokenlist_from_lua(L);
+            break;
+        case P_PAGESATTRIBUTES:
+            pdf_pagesattributes_toks = tokenlist_from_lua(L);
+            break;
+        case P_PDFCATALOG:
+        case P_CATALOG:
+            pdf_catalog_toks = tokenlist_from_lua(L);
+            break;
+        case P_PDFINFO:
+        case P_INFO:
+            pdf_info_toks = tokenlist_from_lua(L);
+            break;
+        case P_PDFNAMES:
+        case P_NAMES:
+            pdf_names_toks = tokenlist_from_lua(L);
+            break;
+        case P_PDFTRAILER:
+        case P_TRAILER:
+            pdf_trailer_toks = tokenlist_from_lua(L);
+            break;
+        case P_H:
+        case P_V:
+            /* can't set |h| and |v| yet */
+        default:
+            lua_rawset(L, -3);
+        }
+    } else {
+        lua_pop(L, 1);          /* ... */
+        lua_rawset(L, -3);
     }
     return 0;
 }
 
+static int l_mapfile(lua_State * L)
+{
+    char *s;
+    const char *st;
+    if (lua_isstring(L, -1) && (st = lua_tostring(L, -1)) != NULL) {
+        s = xstrdup(st);
+        process_map_item(s, MAPFILE);
+        free(s);
+    }
+    return 0;
+}
+
+static int l_mapline(lua_State * L)
+{
+    char *s;
+    const char *st;
+    if (lua_isstring(L, -1) && (st = lua_tostring(L, -1)) != NULL) {
+        s = xstrdup(st);
+        process_map_item(s, MAPLINE);
+        free(s);
+    }
+    return 0;
+}
+
+static int l_pdfmapfile(lua_State * L)
+{
+    luaL_error(L, "pdf.pdfmapfile() is obsolete. Use pdf.mapfile() instead.");
+    return 0;
+}
+
+static int l_pdfmapline(lua_State * L)
+{
+    luaL_error(L, "pdf.pdfmapline() is obsolete. Use pdf.mapline() instead.");
+    return 0;
+}
+
 static const struct luaL_reg pdflib[] = {
-    {"print", luapdfprint},
     {"immediateobj", l_immediateobj},
+    {"mapfile", l_mapfile},
+    {"mapline", l_mapline},
     {"obj", l_obj},
+    {"pdfmapfile", l_pdfmapfile},       /* obsolete */
+    {"pdfmapline", l_pdfmapline},       /* obsolete */
+    {"print", luapdfprint},
     {"registerannot", l_registerannot},
     {"reserveobj", l_reserveobj},
     {NULL, NULL}                /* sentinel */
@@ -633,23 +750,9 @@ static const struct luaL_reg pdflib[] = {
 
 /**********************************************************************/
 
-static void preset_environment(lua_State * L, const parm_struct * p)
-{
-    int i;
-    assert(L != NULL);
-    lua_newtable(L);            /* t */
-    for (i = 1, ++p; p->name != NULL; i++, p++) {
-        assert(i == (int) p->idx);
-        lua_pushstring(L, p->name);     /* k t */
-        lua_pushinteger(L, (int) p->idx);       /* v k t */
-        lua_settable(L, -3);    /* t */
-    }
-    lua_replace(L, LUA_ENVIRONINDEX);   /* - */
-}
-
 int luaopen_pdf(lua_State * L)
 {
-    preset_environment(L, obj_parms);
+    preset_environment(L, pdf_parms);
     luaL_register(L, "pdf", pdflib);
     /* build meta table */
     luaL_newmetatable(L, "pdf_meta");
