@@ -1,6 +1,6 @@
 /* luatex-api.h
-   
-   Copyright 2006-2008 Taco Hoekwater <taco@luatex.org>
+
+   Copyright 2006-2010 Taco Hoekwater <taco@luatex.org>
 
    This file is part of LuaTeX.
 
@@ -17,15 +17,17 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
-/* $Id: luatex-api.h 3078 2009-10-25 22:52:31Z oneiros $ */
+/* $Id: luatex-api.h 3558 2010-03-29 07:21:45Z taco $ */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <stdarg.h>
-#include <../lua51/lua.h>
-#include <../lua51/lauxlib.h>
-#include <../lua51/lualib.h>
+#ifndef LUATEX_API_H
+#  define LUATEX_API_H 1
 
+#  include <stdlib.h>
+#  include <stdio.h>
+#  include <stdarg.h>
+#  include "lua51/lua.h"
+#  include "lua51/lauxlib.h"
+#  include "lua51/lualib.h"
 
 typedef struct LoadS {
     const char *s;
@@ -34,7 +36,8 @@ typedef struct LoadS {
 
 extern lua_State *Luas;
 
-extern void make_table(lua_State * L, char *tab, char *getfunc, char *setfunc);
+extern void make_table(lua_State * L, const char *tab, const char *getfunc,
+                       const char *setfunc);
 
 extern int luac_main(int argc, char *argv[]);
 
@@ -42,14 +45,12 @@ extern int luaopen_tex(lua_State * L);
 
 extern int luaopen_pdf(lua_State * L);
 
-#define LUA_TEXFILEHANDLE               "TEXFILE*"
+#  define LUA_TEXFILEHANDLE               "TEXFILE*"
 
 extern int luaopen_texio(lua_State * L);
 
 extern int luaopen_lang(lua_State * L);
 
-extern void luatex_load_init(int s, LoadS * ls);
-
 extern lua_State *luatex_error(lua_State * L, int fatal);
 
 extern int luaopen_unicode(lua_State * L);
@@ -70,6 +71,7 @@ extern void luatex_socketlua_open(lua_State * L);
 
 extern int luaopen_img(lua_State * L);
 extern int l_new_image(lua_State * L);
+extern int luaopen_epdf(lua_State * L);
 extern int luaopen_mplib(lua_State * L);
 
 extern void open_oslibext(lua_State * L, int safer_option);
@@ -100,30 +102,101 @@ extern void tokenlist_to_luastring(lua_State * L, int p);
 extern int tokenlist_from_lua(lua_State * L);
 
 extern void lua_nodelib_push(lua_State * L);
+extern int nodelib_getdir(lua_State * L, int n);
+
 extern int luaopen_node(lua_State * L);
 extern void nodelist_to_lua(lua_State * L, int n);
 extern int nodelist_from_lua(lua_State * L);
 
-extern int dimen_to_number(lua_State * L, char *s);
+extern int dimen_to_number(lua_State * L, const char *s);
+
+extern int get_command_id(const char *s);
 
 extern void dump_luac_registers(void);
 
 extern void undump_luac_registers(void);
 
+extern int lua_only;
 
-extern void unhide_lua_table(lua_State * lua, char *name, int r);
-extern int hide_lua_table(lua_State * lua, char *name);
+extern void unhide_lua_table(lua_State * lua, const char *name, int r);
+extern int hide_lua_table(lua_State * lua, const char *name);
 
-extern void unhide_lua_value(lua_State * lua, char *name, char *item, int r);
-extern int hide_lua_value(lua_State * lua, char *name, char *item);
+extern void unhide_lua_value(lua_State * lua, const char *name,
+                             const char *item, int r);
+extern int hide_lua_value(lua_State * lua, const char *name, const char *item);
 
 typedef struct command_item_ {
-    char *cmd_name;
+    const char *cmd_name;
     int command_offset;
-    char **commands;
+    const char **commands;
 } command_item;
 
 extern command_item command_names[];
 extern int callback_callbacks_id;
 
 extern void luainterpreter(void);
+
+extern int luabytecode_max;
+extern unsigned int luabytecode_bytes;
+extern int luastate_bytes;
+
+extern int callback_count;
+extern int saved_callback_count;
+
+extern const char *ptexbanner;
+
+/* luastuff.h */
+
+typedef struct {
+    const char *name;           /* parameter name */
+    int idx;                    /* index within img_parms array */
+} parm_struct;
+
+extern void preset_environment(lua_State * L, const parm_struct * p);
+
+extern char *startup_filename;
+extern int safer_option;
+extern int nosocket_option;
+
+extern char *last_source_name;
+extern int last_lineno;
+
+extern int program_name_set;    /* in lkpselib.c */
+
+/* for topenin() */
+extern char **argv;
+extern int argc;
+
+
+extern int loader_C_luatex(lua_State * L, const char *name,
+                           const char *filename);
+extern int loader_Call_luatex (lua_State *L, const char *name,
+			       const char *filename);
+
+
+extern void init_tex_table(lua_State * L);
+
+extern int tex_table_id;
+extern int pdf_table_id;
+extern int token_table_id;
+extern int node_table_id;
+extern int main_initialize(void);
+
+extern int do_run_callback(int special, const char *values, va_list vl);
+extern int lua_traceback(lua_State * L);
+
+extern int luainit;
+
+extern char *luanames[];
+
+extern int ff_get_ttc_index(char *ffname, char *psname);        /* luafontloader/src/luafflib.c */
+extern int ff_createcff(char *, unsigned char **, int *);       /* luafontloader/src/luafflib.c */
+
+extern char *FindResourceTtfFont(char *filename, char *fontname);       /* luafontloader/fontforge/fontforge/macbinary.c */
+
+extern char charsetstr[];       /* from mpdir/psout.w */
+
+extern char **environ;
+
+
+#endif
