@@ -11,17 +11,20 @@
    Makefile rule.  */
 #include "luatex.h"
 
-static const char __svn_version[] =
-    "$Id: luatex.c 3217 2009-12-03 16:52:01Z taco $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/luatex.c $";
+#include "luatex_svnversion.h"
+
+static const char _svn_version[] =
+    "$Id: luatex.c 3588 2010-04-04 06:41:44Z taco $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.60.0/source/texk/web2c/luatexdir/luatex.c $";
 
 #define TeX
 
-int luatex_version = 47;        /* \.{\\luatexversion}  */
+int luatex_svn = luatex_svn_revision;
+int luatex_version = 60;        /* \.{\\luatexversion}  */
 int luatex_revision = '0';      /* \.{\\luatexrevision}  */
 int luatex_date_info = -extra_version_info;     /* the compile date is negated */
-char *luatex_version_string = "beta-0.47.0";
-char *engine_name = "luatex";   /* the name of this engine */
+const char *luatex_version_string = "beta-0.60.0";
+const char *engine_name = "luatex";     /* the name of this engine */
 
 
 
@@ -105,7 +108,7 @@ void mk_shellcmdlist(char *v)
 {
     char **p;
     char *q, *r;
-    int n;
+    unsigned int n;
 
     q = v;
     n = 0;
@@ -120,12 +123,12 @@ void mk_shellcmdlist(char *v)
     }
     if (*q)
         n++;
-    cmdlist = (char **) xmalloc((n + 1) * sizeof(char *));
+    cmdlist = (char **) xmalloc((unsigned) ((n + 1) * sizeof(char *)));
     p = cmdlist;
     q = v;
     while ((r = strchr(q, ',')) != 0) {
         *r = '\0';
-        *p = (char *) xmalloc(strlen(q) + 1);
+        *p = (char *) xmalloc((unsigned) strlen(q) + 1);
         strcpy(*p, q);
         *r = ',';
         r++;
@@ -133,7 +136,7 @@ void mk_shellcmdlist(char *v)
         p++;
     }
     if (*q) {
-        *p = (char *) xmalloc(strlen(q) + 1);
+        *p = (char *) xmalloc((unsigned) strlen(q) + 1);
         strcpy(*p, q);
         p++;
         *p = NULL;
@@ -204,17 +207,19 @@ static int Isspace(char c)
    should get executed.  And we set CMDNAME to its first word; this is
    what is checked against the shell_escape_commands list.  */
 
-int shell_cmd_is_allowed(char **cmd, char **safecmd, char **cmdname)
+int shell_cmd_is_allowed(const char **cmd, char **safecmd, char **cmdname)
 {
     char **p;
     char *buf;
     char *s, *d;
-    int pre, spaces;
+    const char *ss;
+    int pre;
+    unsigned spaces;
     int allow = 0;
 
     /* pre == 1 means that the previous character is a white space
        pre == 0 means that the previous character is not a white space */
-    buf = (char *) xmalloc(strlen(*cmd) + 1);
+    buf = (char *) xmalloc((unsigned) strlen(*cmd) + 1);
     strcpy(buf, *cmd);
     s = buf;
     while (Isspace(*s))
@@ -247,36 +252,37 @@ int shell_cmd_is_allowed(char **cmd, char **safecmd, char **cmdname)
     }
     if (allow == 2) {
         spaces = 0;
-        for (s = *cmd; *s; s++) {
-            if (Isspace(*s))
+        for (ss = *cmd; *ss; ss++) {
+            if (Isspace(*ss))
                 spaces++;
         }
 
         /* allocate enough memory (too much?) */
 #  ifdef WIN32
-        *safecmd = (char *) xmalloc(2 * strlen(*cmd) + 3 + 2 * spaces);
+        *safecmd =
+            (char *) xmalloc(2 * (unsigned) strlen(*cmd) + 3 + 2 * spaces);
 #  else
-        *safecmd = (char *) xmalloc(strlen(*cmd) + 3 + 2 * spaces);
+        *safecmd = (char *) xmalloc((unsigned) strlen(*cmd) + 3 + 2 * spaces);
 #  endif
 
         /* make a safe command line *safecmd */
-        s = *cmd;
-        while (Isspace(*s))
-            s++;
+        ss = *cmd;
+        while (Isspace(*ss))
+            ss++;
         d = *safecmd;
-        while (!Isspace(*s) && *s)
-            *d++ = *s++;
+        while (!Isspace(*ss) && *ss)
+            *d++ = *ss++;
 
         pre = 1;
-        while (*s) {
+        while (*ss) {
             /* Quotation given by a user.  " should always be used; we
                transform it below.  On Unix, if ' is used, simply immediately
                return a quotation error.  */
-            if (*s == '\'') {
+            if (*ss == '\'') {
                 return -1;
             }
 
-            if (*s == '"') {
+            if (*ss == '"') {
                 /* All arguments are quoted as 'foo' (Unix) or "foo" (Windows)
                    before calling system(). Therefore closing QUOTE is necessary
                    if the previous character is not a white space.
@@ -291,51 +297,51 @@ int shell_cmd_is_allowed(char **cmd, char **safecmd, char **cmdname)
                 pre = 0;
                 /* output the quotation mark for the quoted argument */
                 *d++ = QUOTE;
-                s++;
+                ss++;
 
-                while (*s != '"') {
-                    /* Closing quotation mark is missing */
-                    if (*s == '\0')
+                while (*ss != '"') {
+                    /* Illegal use of ', or closing quotation mark is missing */
+                    if (*ss == '\'' || *ss == '\0')
                         return -1;
 #  ifdef WIN32
-                    if (char_needs_quote(*s))
+                    if (char_needs_quote(*ss))
                         *d++ = '^';
 #  endif
-                    *d++ = *s++;
+                    *d++ = *ss++;
                 }
 
                 /* Closing quotation mark will be output afterwards, so
                    we do nothing here */
-                s++;
+                ss++;
 
                 /* The character after the closing quotation mark
                    should be a white space or NULL */
-                if (!Isspace(*s) && *s)
+                if (!Isspace(*ss) && *ss)
                     return -1;
 
                 /* Beginning of a usual argument */
-            } else if (pre == 1 && !Isspace(*s)) {
+            } else if (pre == 1 && !Isspace(*ss)) {
                 pre = 0;
                 *d++ = QUOTE;
 #  ifdef WIN32
-                if (char_needs_quote(*s))
+                if (char_needs_quote(*ss))
                     *d++ = '^';
 #  endif
-                *d++ = *s++;
+                *d++ = *ss++;
                 /* Ending of a usual argument */
 
-            } else if (pre == 0 && Isspace(*s)) {
+            } else if (pre == 0 && Isspace(*ss)) {
                 pre = 1;
                 /* Closing quotation mark */
                 *d++ = QUOTE;
-                *d++ = *s++;
+                *d++ = *ss++;
             } else {
                 /* Copy a character from *cmd to *safecmd. */
 #  ifdef WIN32
-                if (char_needs_quote(*s))
+                if (char_needs_quote(*ss))
                     *d++ = '^';
 #  endif
-                *d++ = *s++;
+                *d++ = *ss++;
             }
         }
         /* End of the command line */
@@ -367,10 +373,12 @@ int runsystem(char *cmd)
     }
 
     /* If restrictedshell == 0, any command is allowed. */
-    if (restrictedshell == 0)
+    if (restrictedshell == 0) {
         allow = 1;
-    else
-        allow = shell_cmd_is_allowed(&cmd, &safecmd, &cmdname);
+    } else {
+        const char *thecmd = cmd;
+        allow = shell_cmd_is_allowed(&thecmd, &safecmd, &cmdname);
+    }
 
     if (allow == 1)
         (void) system(cmd);
@@ -396,7 +404,7 @@ char **argv;
 int argc;
 
 /* The C version of what might wind up in |TEX_format_default|.  */
-const_string dump_name;
+string dump_name;
 
 /* The C version of the jobname, if given. */
 const_string c_job_name;
@@ -454,7 +462,7 @@ void topenin(void)
             /* Don't use strcat, since in Aleph the buffer elements aren't
                single bytes.  */
             while (*ptr) {
-                buffer[k++] = *(ptr++);
+                buffer[k++] = (packed_ASCII_code) * (ptr++);
             }
             buffer[k++] = ' ';
         }
@@ -666,7 +674,7 @@ string normalize_quotes(const_string name, const_string mesg)
     boolean quoted = false;
     boolean must_quote = (strchr(name, ' ') != NULL);
     /* Leave room for quotes and NUL. */
-    string ret = (string) xmalloc(strlen(name) + 3);
+    string ret = (string) xmalloc((unsigned) strlen(name) + 3);
     string p;
     const_string q;
     p = ret;
@@ -724,9 +732,7 @@ static RETSIGTYPE catch_interrupt(int arg)
    ``Get the first line of input and prepare to start''), this is as
    good a place as any.  */
 
-void
-get_date_and_time(integer * minutes, integer * day,
-                  integer * month, integer * year)
+void get_date_and_time(int *minutes, int *day, int *month, int *year)
 {
     time_t myclock = time((time_t *) 0);
     struct tm *tmptr = localtime(&myclock);
@@ -770,13 +776,13 @@ get_date_and_time(integer * minutes, integer * day,
 /*
  Getting a high resolution time.
  */
-void get_seconds_and_micros(integer * seconds, integer * micros)
+void get_seconds_and_micros(int *seconds, int *micros)
 {
 #if defined (HAVE_GETTIMEOFDAY)
     struct timeval tv;
     gettimeofday(&tv, NULL);
-    *seconds = tv.tv_sec;
-    *micros = tv.tv_usec;
+    *seconds = (int)tv.tv_sec;
+    *micros = (int)tv.tv_usec;
 #elif defined (HAVE_FTIME)
     struct timeb tb;
     ftime(&tb);
@@ -784,7 +790,7 @@ void get_seconds_and_micros(integer * seconds, integer * micros)
     *micros = tb.millitm * 1000;
 #else
     time_t myclock = time((time_t *) NULL);
-    *seconds = myclock;
+    *seconds = (int) myclock;
     *micros = 0;
 #endif
 }
@@ -792,12 +798,12 @@ void get_seconds_and_micros(integer * seconds, integer * micros)
 /*
   Generating a better seed numbers
   */
-integer getrandomseed()
+int getrandomseed()
 {
 #if defined (HAVE_GETTIMEOFDAY)
     struct timeval tv;
     gettimeofday(&tv, NULL);
-    return (tv.tv_usec + 1000000 * tv.tv_usec);
+    return (int)(tv.tv_usec + 1000000 * tv.tv_usec);
 #elif defined (HAVE_FTIME)
     struct timeb tb;
     ftime(&tb);
@@ -821,7 +827,7 @@ boolean input_line(FILE * f)
     /* Recognize either LF or CR as a line terminator.  */
     last = first;
     while (last < buf_size && (i = getc(f)) != EOF && i != '\n' && i != '\r')
-        buffer[last++] = i;
+        buffer[last++] = (packed_ASCII_code) i;
 
     if (i == EOF && errno != EINTR && last == first)
         return false;
@@ -866,7 +872,7 @@ static const_string edit_value = EDITOR;
 
 void
 calledit(packedASCIIcode * filename,
-         poolpointer fnstart, integer fnlength, integer linenumber)
+         poolpointer fnstart, int fnlength, int linenumber)
 {
     char *temp, *command;
     char c;
@@ -887,7 +893,9 @@ calledit(packedASCIIcode * filename,
 
     /* Construct the command string.  The `11' is the maximum length an
        integer might be.  */
-    command = (string) xmalloc(strlen(edit_value) + fnlength + 11);
+    command =
+        (string) xmalloc((unsigned) strlen(edit_value) + (unsigned) fnlength +
+                         11);
 
     /* So we can construct it as we go.  */
     temp = command;
@@ -908,7 +916,7 @@ calledit(packedASCIIcode * filename,
                 if (sdone)
                     FATAL("call_edit: `%%s' appears twice in editor command");
                 for (i = 0; i < fnlength; i++)
-                    *temp++ = Xchr(filename[i]);
+                    *temp++ = (char) Xchr(filename[i]);
                 sdone = 1;
                 break;
 
@@ -1031,9 +1039,10 @@ void do_dump(char *p, int item_size, int nitems, FILE * out_file)
     swap_items(p, nitems, item_size);
 #endif
 
-    if (fwrite(p, item_size, nitems, out_file) != (unsigned) nitems) {
-        fprintf(stderr, "! Could not write %d %d-byte item(s).\n",
-                nitems, item_size);
+    if (fwrite(p, (size_t) item_size, (size_t) nitems, out_file) !=
+        (unsigned) nitems) {
+        fprintf(stderr, "! Could not write %d %d-byte item(s).\n", nitems,
+                item_size);
         uexit(1);
     }
 
@@ -1049,7 +1058,8 @@ void do_dump(char *p, int item_size, int nitems, FILE * out_file)
 
 void do_undump(char *p, int item_size, int nitems, FILE * in_file)
 {
-    if (fread(p, item_size, nitems, in_file) != (size_t) nitems)
+    if (fread(p, (size_t) item_size, (size_t) nitems, in_file) !=
+        (size_t) nitems)
         FATAL2("Could not undump %d %d-byte item(s)", nitems, item_size);
 
 #if !defined (WORDS_BIGENDIAN) && !defined (NO_DUMP_SHARE)
@@ -1057,30 +1067,6 @@ void do_undump(char *p, int item_size, int nitems, FILE * in_file)
 #endif
 }
 
-/* Look up VAR_NAME in texmf.cnf; assign either the value found there or
-   DFLT to *VAR.  */
-
-void setupboundvariable(integer * var, const_string var_name, integer dflt)
-{
-    string expansion = kpse_var_value(var_name);
-    *var = dflt;
-
-    if (expansion) {
-        integer conf_val = atoi(expansion);
-        /* It's ok if the cnf file specifies 0 for extra_mem_{top,bot}, etc.
-           But negative numbers are always wrong.  */
-        if (conf_val < 0 || (conf_val == 0 && dflt > 0)) {
-            fprintf(stderr,
-                    "%s: Bad value (%ld) in texmf.cnf for %s, keeping %ld.\n",
-                    program_invocation_name,
-                    (long) conf_val, var_name + 1, (long) dflt);
-        } else {
-            *var = conf_val;    /* We'll make further checks later.  */
-        }
-        free(expansion);
-    }
-}
-
 
 /* Get the job name to be used, which may have been set from the
    command line. */
