@@ -17,7 +17,7 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
-/* $Id: dumpdata.h 3125 2009-11-11 09:41:44Z taco $ */
+/* $Id: dumpdata.h 3388 2010-01-26 10:59:04Z taco $ */
 
 #ifndef DUMPDATA_H
 #  define DUMPDATA_H
@@ -27,7 +27,7 @@ extern str_number format_name;  /* principal file name */
 extern FILE *fmt_file;          /* for input or output of format information */
 
 extern void store_fmt_file(void);
-extern boolean load_fmt_file(char *);
+extern boolean load_fmt_file(const char *);
 
 /* (Un)dumping.  These are called from the change file.  */
 #  define        dump_things(base, len) \
@@ -88,7 +88,7 @@ extern void do_zundump(char *, int, int, FILE *);
 #  define	dump_int(x)							\
   do									\
     {									\
-      integer x_val = (x);						\
+      int x_val = (x);					     	        \
       generic_dump (x_val);						\
     }									\
   while (0)
@@ -100,7 +100,7 @@ extern void do_zundump(char *, int, int, FILE *);
 #    define undump_int(x)							\
   do									\
     {									\
-      integer x_val;							\
+      int x_val;							\
       generic_undump (x_val);						\
       x = x_val;							\
     }									\
